#include <J3ML/Algorithm/Bezier.hpp>

namespace J3ML::Algorithm
{
    using namespace J3ML::LinearAlgebra;
    Vector2 BezierNormal(float t, const Vector2 &p0, const Vector2 &p1,
                                  const Vector2 &p2, const Vector2 &p3) {
        auto derived = BezierDerivative(t, p0, p1, p2, p3);
        return derived.Normalized();
    }

    Vector2 BezierDerivative(float t, const Vector2 &p0, const Vector2 &p1, const Vector2 &p2, const Vector2 &p3) {
        return 3 * Square(1 - t) * (p1 - p0) + 6 * (1 - t) * t * (p2 - p1) + 3 * Square(t) * (p3 - p2);
    }

    Vector2 Bezier(float t, const Vector2 &p0, const Vector2 &p1, const Vector2 &p2, const Vector2 &p3) {
        return {Bezier(t, p0.x, p1.x, p2.x, p3.x), Bezier(t, p0.y, p1.y, p2.y, p3.y)};
    }
}

